<?php

namespace FiloBlu\Job\Validator;

use Magento\Framework\App\RequestInterface;
use Zend\Validator\ValidatorInterface;
use Zend_Validate_Exception;
use Zend_Validate_Interface;

/**
 * Class Request
 * @package FiloBlu\Job\Validator
 */
class Request implements Zend_Validate_Interface
{
    /**
     * @var ValidatorInterface
     */
    protected $mimeValidator;
    /**
     * @var Zend_Validate_Interface
     */
    protected $sizeValidator;
    /**
     * @var ValidatorInterface
     */
    protected $notEmptyValidator;
    /**
     * @var ValidatorInterface
     */
    protected $emailAddressValidator;
    /**
     * @var array
     */
    protected $messages = [];
    /**
     * @var string
     */
    protected $attachmentName;

    /**
     * Attachment constructor.
     * @param ValidatorInterface $mimeValidator
     * @param Zend_Validate_Interface $sizeValidator
     * @param ValidatorInterface $notEmptyValidator
     * @param ValidatorInterface $emailAddressValidator
     * @param string $attachmentName
     */
    public function __construct(
        ValidatorInterface $mimeValidator,
        Zend_Validate_Interface $sizeValidator,
        ValidatorInterface $notEmptyValidator,
        ValidatorInterface $emailAddressValidator,
        $attachmentName
    ) {
        $this->mimeValidator = $mimeValidator;
        $this->sizeValidator = $sizeValidator;
        $this->notEmptyValidator = $notEmptyValidator;
        $this->emailAddressValidator = $emailAddressValidator;
        $this->attachmentName = $attachmentName;
    }

    /**
     * Returns true if and only if $value meets the validation requirements
     *
     * If $value fails validation, then this method returns false, and
     * getMessages() will return an array of messages that explain why the
     * validation failed.
     *
     * @param RequestInterface $value
     * @return boolean
     * @throws Zend_Validate_Exception If validation of $value is impossible
     */
    public function isValid($value)
    {
        $this->messages = [];
        $email = $value->getParam('email');
        $result = true;

        if (!$this->notEmptyValidator->isValid($email)) {
            $this->messages = array_merge($this->messages, $this->notEmptyValidator->getMessages());
            $result = false;
        }

        if (!$this->emailAddressValidator->isValid($email)) {
            $this->messages = array_merge($this->messages, $this->emailAddressValidator->getMessages());
            $result = false;
        }

        $parameters = $value->getFiles($this->attachmentName);

        $file = $parameters['tmp_name'];
        if (!$this->mimeValidator->isValid($file)) {
            $this->messages = array_merge($this->messages, $this->mimeValidator->getMessages());
            $result = false;
        }

        if (!$this->sizeValidator->isValid($file)) {
            $this->messages = array_merge($this->messages, $this->sizeValidator->getMessages());
            $result = false;
        }

        return $result;
    }

    /**
     * Returns an array of messages that explain why the most recent isValid()
     * call returned false. The array keys are validation failure message identifiers,
     * and the array values are the corresponding human-readable message strings.
     *
     * If isValid() was never called or if the most recent isValid() call
     * returned true, then this method returns an empty array.
     *
     * @return array
     */
    public function getMessages()
    {
        return $this->messages;
    }
}
